from prism._utils.exceptions import PrismValueError
from ..._common.const import (
    EstimateDataComponentType as _EstimateDataComponentType,
    PeriodType,
)
from .._req_builder._list import _list_dataitem_estimate
from ..._prismcomponent.datacomponent import (
    _Actual,
    _Consensus,
    _Growth,
    _Guidance,
    _Revision,
    _Surprise,
)
from ..._utils.validate_utils import _validate_args


__all__ = [
    'actual',
    'consensus',
    'growth',
    'guidance',
    'revision',
    'surprise',
    'dataitems',
    'actual_dataitems',
    'consensus_dataitems',
    'growth_dataitems',
    'guidance_dataitems',
    'revision_dataitems',
    'surprise_dataitems',
    'other_dataitems',
]


@_validate_args
def _estimate_builder(
    datacomponentclass,
    dataitemid: int,
    periodtype: str,
    periodout: int = 0,
    currency: str = None,
    package: str = None,
):
    return datacomponentclass(
        dataitemid=dataitemid,
        periodtype=periodtype,
        periodout=periodout,
        package=package,
        currency=currency,
    )


def actual(
    dataitemid: int,
    periodtype: str,
    periodout: int = 0,
    currency: str = 'report',
    package: str = None,
):
    return _estimate_builder(
        _Actual,
        dataitemid=dataitemid,
        periodtype=periodtype,
        periodout=periodout,
        currency=currency,
        package=package,
    )


def consensus(
    dataitemid: int,
    periodtype: str,
    periodout: int = 0,
    currency: str = 'report',
    package: str = None,
):
    if periodtype in ["LTM"]:
        raise PrismValueError(
            f"Balance Sheet cannot take {periodtype} as periodtype.",
            valid_list=PeriodType,
            invalids=["YTD", "LTM", "NTM", "Non-Periodic"],
        )
    return _estimate_builder(
        _Consensus,
        dataitemid=dataitemid,
        periodtype=periodtype,
        periodout=periodout,
        currency=currency,
        package=package,
    )


def growth(
    dataitemid: int,
    periodtype: str,
    periodout: int = 0,
    currency: str = 'report',
    package: str = None,
):
    if periodtype in ["LTM"]:
        raise PrismValueError(
            f"Balance Sheet cannot take {periodtype} as periodtype.",
            valid_list=PeriodType,
            invalids=["YTD", "LTM", "NTM", "Non-Periodic"],
        )
    return _estimate_builder(
        _Growth,
        dataitemid=dataitemid,
        periodtype=periodtype,
        periodout=periodout,
        currency=currency,
        package=package,
    )


def guidance(
    dataitemid: int,
    periodtype: str,
    periodout: int = 0,
    currency: str = 'report',
    package: str = None,
):
    if periodtype in ["LTM"]:
        raise PrismValueError(
            f"Balance Sheet cannot take {periodtype} as periodtype.",
            valid_list=PeriodType,
            invalids=["YTD", "LTM", "NTM", "Non-Periodic"],
        )
    return _estimate_builder(
        _Guidance,
        dataitemid=dataitemid,
        periodtype=periodtype,
        periodout=periodout,
        currency=currency,
        package=package,
    )


def revision(
    dataitemid: int,
    periodtype: str,
    periodout: int = 0,
    currency: str = 'report',
    package: str = None,
):
    if periodtype in ["LTM"]:
        raise PrismValueError(
            f"Balance Sheet cannot take {periodtype} as periodtype.",
            valid_list=PeriodType,
            invalids=["YTD", "LTM", "NTM", "Non-Periodic"],
        )
    return _estimate_builder(
        _Revision,
        dataitemid=dataitemid,
        periodtype=periodtype,
        periodout=periodout,
        currency=currency,
        package=package,
    )


def surprise(
    dataitemid: int,
    periodtype: str,
    periodout: int = 0,
    currency: str = 'report',
    package: str = None,
):
    return _estimate_builder(
        _Surprise,
        dataitemid=dataitemid,
        periodtype=periodtype,
        periodout=periodout,
        currency=currency,
        package=package,
    )


@_validate_args
def dataitems(search: str = None, package: str = None):
    return _list_dataitem_estimate(None, search, package)


@_validate_args
def actual_dataitems(search: str = None, package: str = None):
    return _list_dataitem_estimate(_EstimateDataComponentType.ACTUAL, search, package)


@_validate_args
def consensus_dataitems(search: str = None, package: str = None):
    return _list_dataitem_estimate(_EstimateDataComponentType.CONSENSUS, search, package)


@_validate_args
def growth_dataitems(search: str = None, package: str = None):
    return _list_dataitem_estimate(_EstimateDataComponentType.GROWTH, search, package)


@_validate_args
def guidance_dataitems(search: str = None, package: str = None):
    return _list_dataitem_estimate(_EstimateDataComponentType.GUIDANCE, search, package)


@_validate_args
def revision_dataitems(search: str = None, package: str = None):
    return _list_dataitem_estimate(_EstimateDataComponentType.REVISION, search, package)


@_validate_args
def surprise_dataitems(search: str = None, package: str = None):
    return _list_dataitem_estimate(_EstimateDataComponentType.SURPRISE, search, package)
