from struct import pack
from prism._utils.exceptions import PrismValueError
from ..._common.const import (
    FinancialDataComponentType as _FinancialDataComponentType,
    PeriodType,
)
from .._req_builder._list import _list_dataitem_financial
from ..._prismcomponent.datacomponent import (
    _BalanceSheet,
    _IncomeStatement,
    _CashFlow,
    _DPS,
    _EPS,
)
from ..._utils.validate_utils import _validate_args


__all__ = [
    "balance_sheet",
    "income_statement",
    "cash_flow",
    "dps",
    "eps",
    "other",
    "dataitems",
    "balance_sheet_dataitems",
    "income_statement_dataitems",
    "cash_flow_dataitems",
    "other_dataitems",
]


@_validate_args
def _build_financial(
    datacomponentclass,
    dataitemid: int,
    periodtype: str,
    currency: str = None,
    preliminary: bool = True,
    package: str = None,
):
    if periodtype in ["NTM", "Non-Periodic"]:
        raise PrismValueError(
            f"Financial components cannot take {periodtype} as periodtype.", valid_list=PeriodType, invalids=["NTM", "Non-Periodic"]
        )
    return datacomponentclass(
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def balance_sheet(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = "CIQ Premium Financials",
):
    if periodtype in ["YTD", "LTM"]:
        raise PrismValueError(
            f"Balance Sheet cannot take {periodtype} as periodtype.", valid_list=PeriodType, invalids=["YTD", "LTM", "NTM", "Non-Periodic"]
        )
    return _build_financial(
        _BalanceSheet,
        dataitemid=dataitemid,
        periodtype=periodtype,
        currency=currency,
        preliminary=preliminary,
        package=package,
    )


def cash_flow(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = "CIQ Premium Financials",
):
    return _build_financial(
        _CashFlow,
        dataitemid=dataitemid,
        periodtype=periodtype,
        currency=currency,
        preliminary=preliminary,
        package=package,
    )


def dps(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = "CIQ Premium Financials",
):
    return _build_financial(
        _DPS,
        dataitemid=dataitemid,
        periodtype=periodtype,
        currency=currency,
        preliminary=preliminary,
        package=package,
    )


def eps(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = "CIQ Premium Financials",
):
    return _build_financial(
        _EPS,
        dataitemid=dataitemid,
        periodtype=periodtype,
        currency=currency,
        preliminary=preliminary,
        package=package,
    )


def income_statement(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = "CIQ Premium Financials",
):
    return _build_financial(
        _IncomeStatement,
        dataitemid=dataitemid,
        periodtype=periodtype,
        currency=currency,
        preliminary=preliminary,
        package=package,
    )


@_validate_args
def dataitems(search: str = None, package: str = None):
    return _list_dataitem_financial(None, search, package)


@_validate_args
def balance_sheet_dataitems(search: str = None, package: str = None):
    return _list_dataitem_financial(_FinancialDataComponentType.BALANCE_SHEET, search, package)


@_validate_args
def income_statement_dataitems(search: str = None, package: str = None):
    return _list_dataitem_financial(_FinancialDataComponentType.INCOME_STATEMENT, search, package)


@_validate_args
def cash_flow_dataitems(search: str = None, package: str = None):
    return _list_dataitem_financial(_FinancialDataComponentType.CASH_FLOW, search, package)


@_validate_args
def eps_dataitems(search: str = None, package: str = None):
    return _list_dataitem_financial(_FinancialDataComponentType.EPS, search, package)


@_validate_args
def dps_dataitems(search: str = None, package: str = None):
    return _list_dataitem_financial(_FinancialDataComponentType.DPS, search, package)
