from prism._utils.exceptions import PrismTypeError
from .._req_builder._list import _list_dataitem_index
from ..._prismcomponent.datacomponent import _IndexLevel, _IndexShare, _IndexWeight
from ..._common.const import INDEXLEVELTYPE2ID, IndexDataComponentType
from ..._utils import _validate_args

__all__ = ["share", "weight", "level", "dataitems"]


@_validate_args
def share(dataitemid: int):
    return _IndexShare(dataitemid=dataitemid)


@_validate_args
def weight(dataitemid: int):
    return _IndexWeight(dataitemid=dataitemid)


@_validate_args
def level(dataitemid: int, leveltype=None):
    leveltypeid = None
    if leveltype is not None:
        if isinstance(leveltype, str):
            leveltypeid = INDEXLEVELTYPE2ID.get(leveltype)
            if leveltypeid is None:
                raise PrismTypeError(f"Level Type should be one of: {list(INDEXLEVELTYPE2ID.keys())}")
        elif isinstance(leveltype, int):
            leveltypeid = leveltype
            if leveltype not in INDEXLEVELTYPE2ID.values():
                raise PrismTypeError(f"Level Type ID should be one of: {list(INDEXLEVELTYPE2ID.values())}")
    return _IndexLevel(dataitemid=dataitemid, leveltypeid=leveltypeid)


@_validate_args
def share_dataitems(search: str = None, package: str = None):
    ret = _list_dataitem_index(
        datacomponent=IndexDataComponentType.SHARE,
        search=search,
        package=package,
    )
    ret = ret.drop(["dataitemdescription"], axis=1, errors="ignore")
    return ret


@_validate_args
def weight_dataitems(search: str = None, package: str = None):
    ret = _list_dataitem_index(
        datacomponent=IndexDataComponentType.WEIGHT,
        search=search,
        package=package,
    )
    ret = ret.drop(["dataitemdescription"], axis=1, errors="ignore")
    return ret


@_validate_args
def level_dataitems(search: str = None, package: str = None):
    ret = _list_dataitem_index(datacomponent=IndexDataComponentType.LEVEL, search=search, package=package)
    ret = ret.drop(["dataitemdescription"], axis=1, errors="ignore")
    return ret


@_validate_args
def universe_dataitems(search: str = None, package: str = None):
    ret = _list_dataitem_index(datacomponent="Universe", search=search, package=package)
    ret = ret.drop(["dataitemdescription"], axis=1, errors="ignore")
    return ret


@_validate_args
def portfolio_dataitems(search: str = None, package: str = None):
    ret = _list_dataitem_index(datacomponent="Portfolio", search=search, package=package)
    ret = ret.drop(["dataitemdescription"], axis=1, errors="ignore")
    return ret


@_validate_args
def dataitems(search: str = None, package: str = None):
    ret = _list_dataitem_index(datacomponent=None, search=search, package=package)
    ret = ret.drop(["dataitemdescription"], axis=1, errors="ignore")
    return ret
