from prism._utils.exceptions import PrismValueError
from ..._common.const import (
    IndustryComponentType as _IndustryComponentType,
    PeriodType,
)
from .._req_builder._list import _list_dataitem_industry
from ..._prismcomponent.datacomponent import (
    _Airlines,
    _Bank,
    _CapitalMarket,
    _FinancialServices,
    _Healthcare,
    _Homebuilders,
    _HotelandGaming,
    _Insurance,
    _InternetMedia,
    _ManagedCare,
    _MetalsandMining,
    _OilandGas,
    _Pharmaceutical,
    _RealEstate,
    _Restaurant,
    _Retail,
    _Semiconductors,
    _Telecom,
    _Utility,
)
from ..._utils.validate_utils import _validate_args


__all__ = [
    'airlines',
    'bank',
    'capital_market',
    'financial_services',
    'healthcare',
    'homebuilders',
    'hotel',
    'insurance',
    'internet',
    'managed_care',
    'mining',
    'oil',
    'biopharma',
    'real_estate',
    'restaurant',
    'retail',
    'semiconductors',
    'telecom',
    'utility',
    'airlines_dataitems',
    'bank_dataitems',
    'capital_market_dataitems',
    'financial_services_dataitems',
    'healthcare_dataitems',
    'homebuilders_dataitems',
    'hotel_dataitems',
    'insurance_dataitems',
    'internet_dataitems',
    'managed_care_dataitems',
    'mining_dataitems',
    'oil_dataitems',
    'biopharma_dataitems',
    'real_estate_dataitems',
    'restaurant_dataitems',
    'retail_dataitems',
    'semiconductors_dataitems',
    'telecom_dataitems',
    'utility_dataitems',
]


@_validate_args
def _build_industry(
    datacomponentclass,
    dataitemid: int,
    periodtype: str,
    currency: str = None,
    preliminary: bool = True,
    package: str = None,
):
    if periodtype in ["NTM", "Non-Periodic"]:
        raise PrismValueError(
            f"Financial components cannot take {periodtype} as periodtype.",
            valid_list=PeriodType,
            invalids=["NTM", "Non-Periodic"],
        )
    return datacomponentclass(
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def airlines(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _Airlines,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def bank(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _Bank,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def capital_market(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _CapitalMarket,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def financial_services(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _FinancialServices,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def healthcare(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _Healthcare,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def homebuilders(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _Homebuilders,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def hotel(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _HotelandGaming,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def insurance(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _Insurance,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def internet(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _InternetMedia,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def managed_care(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _ManagedCare,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def mining(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _MetalsandMining,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def oil(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _OilandGas,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def biopharma(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _Pharmaceutical,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def real_estate(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _RealEstate,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def restaurant(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _Restaurant,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def retail(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _Retail,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def semiconductors(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _Semiconductors,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def telecom(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _Telecom,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def utility(
    dataitemid: int,
    periodtype: str,
    currency: str = 'report',
    preliminary: bool = True,
    package: str = None,
):
    return _build_industry(
        _Utility,
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


@_validate_args
def dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(None, search, package)


@_validate_args
def airlines_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.AIRLINES, search, package)


@_validate_args
def bank_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.BANK, search, package)


@_validate_args
def capital_market_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.CAPITAL_MARKET, search, package)


@_validate_args
def financial_services_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.FINAICIAL_SERVICES, search, package)


@_validate_args
def healthcare_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.HEALTHCARE, search, package)


@_validate_args
def homebuilders_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.HOMBUILDERS, search, package)


@_validate_args
def hotel_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.HOTEL_AND_GAMING, search, package)


@_validate_args
def insurance_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.INSURANCE, search, package)


@_validate_args
def internet_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.INTERNET_MEDIA, search, package)


@_validate_args
def managed_care_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.MANAGED_CARE, search, package)


@_validate_args
def mining_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.METALS_AND_MINING, search, package)


@_validate_args
def oil_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.OIL_AND_GAS, search, package)


@_validate_args
def biopharma_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.PHARMA, search, package)


@_validate_args
def real_estate_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.REAL_ESTATE, search, package)


@_validate_args
def restaurant_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.RESTAURANT, search, package)


@_validate_args
def retail_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.RETAIL, search, package)


@_validate_args
def semiconductors_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.SEMICONDUCTORS, search, package)


@_validate_args
def telecom_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.TELECOM, search, package)


@_validate_args
def utility_dataitems(search: str = None, package: str = None):
    return _list_dataitem_industry(_IndustryComponentType.UTILITY, search, package)
