import numbers

from prism._utils.exceptions import PrismTypeError

from .._prismcomponent.prismcomponent import _PrismComponent, __PrismValue, _functioncomponent_builder
from .._utils import _validate_args


__all__ = ["min", "max", "mean", "std"]


def _fn_builder(component_name, component_args, *cmpts):
    cmpts_ = []
    for c in cmpts:
        if isinstance(c, _PrismComponent):
            cmpts_.append(c)
        elif isinstance(c, numbers.Real):
            cmpts_.append(__PrismValue(data=c))
        else:
            raise PrismTypeError(f"Incompatible type for {component_name}: {type(c)}")

    return _functioncomponent_builder(component_name, component_args, *cmpts_)


@_validate_args
def min(cmpts: list):
    return _fn_builder("min", {}, *cmpts)


@_validate_args
def max(cmpts: list):
    return _fn_builder("max", {}, *cmpts)


@_validate_args
def mean(cmpts: list):
    return _fn_builder("mean", {}, *cmpts)


@_validate_args
def std(cmpts: list):
    return _fn_builder("std", {}, *cmpts)
