from ..._common.config import *
from ..._common.const import *
from ..._utils import _validate_args, get, delete, post


__all__ = ['list_job', 'get_job', 'delete_job', 'strategy_backtest_jobs', 'factor_backtest_jobs', 'screen_jobs', 'extract_job']


@_validate_args
def list_job(
    tasktype: str = None,
    jobstatus: str = None,
    startrange: str = None,
    endrange: str = None,
):
    param = {'jobstatus': jobstatus, 'tasktype': tasktype, 'startrange': startrange, 'endrange': endrange}
    return get(URL_JOBS, param)


@_validate_args
def get_job(jobid: int):
    return get(f'{URL_JOBS}/id/{jobid}', None)


@_validate_args
def strategy_backtest_jobs():
    return get(URL_JOBS + '/type/strategy_backtest', None)


@_validate_args
def factor_backtest_jobs():
    return get(URL_JOBS + '/type/factor_backtest', None)


@_validate_args
def screen_jobs():
    return get(URL_JOBS + '/type/screen', None)


@_validate_args
def delete_job(jobid: int):
    ret = delete(URL_JOBS + f'/{jobid}')
    print(ret['message'])
    return ret


@_validate_args
def extract_job(jobid: int, return_code=False):
    code = get(URL_JOBS + f'/extract/{jobid}', {'dialect': 'python'})
    if return_code:
        return code
    else:
        print(code)


@_validate_args
def rename_job(jobid: int, jobname: str):
    return post(URL_JOBS + f'/jobname/{jobid}', body=jobname)


@_validate_args
def add_job_description(jobid: int, description: str):
    return post(URL_JOBS + f'/description/{jobid}', body=description)