import copy
import traceback
from abc import ABC
from uuid import UUID

from .._utils.exceptions import PrismNotFoundError
from .._common.const import PrismComponentType


class _AbstractPrismComponent(ABC):
    """
    Args:
        query(dict): incl. component_type, component_name, component_args, children
    """

    def __init__(
        self,
        *,
        component_type: str,
        component_name: str,
        component_args: dict = {},
        children: list,
        nodeid: UUID = None,
        query_name: str = None,
    ):
        # if not hasattr(self, '_func_name'): self._func_name = func_name
        if component_type == PrismComponentType.FUNCTION_COMPONENT:
            if not hasattr(self, "_component_name"):
                self._component_name = component_name
        self._query = {
            "component_type": component_type,
            "component_name": component_name,
            "component_args": component_args,
            "children": children,
            "nodeid": nodeid,
        }
        (_, _, _, text) = traceback.extract_stack()[-2]
        self.text = text.strip()
        # def_name = text[:text.find('=')].strip()

    def __repr__(self):
        self.query(verbose=False)
        return "Query Structure"

    def _dict_to_tree(self, query: dict, verbose: bool, depth: int = 0):
        if not verbose:
            try:
                print("\t" * depth, "====", query["component_name"])
                for c in query["children"]:
                    self._dict_to_tree(c, False, depth + 1)
            except:
                pass
        else:
            try:
                print(
                    "\t" * depth,
                    "====",
                    query["component_name"],
                )
                if len(query["component_args"]) > 0:
                    print(
                        "\t" * (depth + 1),
                        "parameters: {",
                    )
                    for k, v in query["component_args"].items():
                        if "_dataquery" in k:
                            print("\t" * (depth + 2), k.split("_dataquery")[0], ":")
                            if isinstance(v, list):
                                for d in v:
                                    print(self._dict_to_tree(d, True, depth + 2))
                            else:
                                print(self._dict_to_tree(v, True, depth + 2))
                        else:
                            print("\t" * (depth + 2), k, ":", v)
                    print("\t" * (depth + 2), "}")
                else:
                    print("\t" * (depth + 1), "parameters: {}")
                for idx, c in enumerate(query["children"]):
                    if (self._component_name == "map") & (idx != 0):
                        break
                    self._dict_to_tree(c, True, depth + 1)
            except:
                pass

    def copy(self):
        return copy.deepcopy(self)

    def query(self, verbose: bool = True):
        self._dict_to_tree(self._query, verbose)
